package com.indiscale.fdo.manager.service.mock;

import com.indiscale.fdo.DefaultManager;
import com.indiscale.fdo.DefaultProfileRegistry;
import com.indiscale.fdo.DefaultRepositoryRegistry;
import com.indiscale.fdo.api.Data;
import com.indiscale.fdo.api.DigitalObject;
import com.indiscale.fdo.api.FDO;
import com.indiscale.fdo.api.FdoProfile;
import com.indiscale.fdo.api.Metadata;
import com.indiscale.fdo.api.ProfileValidator;
import com.indiscale.fdo.api.RepositoryConnection;
import com.indiscale.fdo.api.UnknownProfileException;
import com.indiscale.fdo.api.UnknownRepositoryTypeException;
import com.indiscale.fdo.api.ValidationError;
import com.indiscale.fdo.api.ValidationException;
import com.indiscale.fdo.api.ValidationResult;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

class NoOpValidator implements ProfileValidator<FDO> {

  @Override
  public ValidationResult validate(FDO t) {
    return new ValidationResult() {

      @Override
      public boolean isValid() {
        return true;
      }

      @Override
      public List<ValidationError> getErrors() {
        return Collections.emptyList();
      }
    };
  }
}

public class MockManager extends DefaultManager {

  private static Map<String, DigitalObject> pidRegistry = new HashMap<>();

  public MockManager() {
    MockRepositoryFactory mock = new MockRepositoryFactory();
    DefaultRepositoryRegistry registry = this.getRepositoryRegistry();
    registry.registerRepositoryType(mock);
    try {
      registry.registerRepository(mock.createMockConfig("mock-repo-1"));
      registry.registerRepository(mock.createMockConfig("mock-repo-2"));
      registry.registerRepository(mock.createMockConfig("mock-repo-3"));
    } catch (UnknownRepositoryTypeException e) {
      e.printStackTrace();
    }
    DefaultProfileRegistry<FdoProfile> profileRegistry = getProfileRegistry();
    profileRegistry.registerProfile(createFdoProfile("mock-profile-1"));
    try {
      setDefaultProfile("mock-profile-1");
    } catch (UnknownProfileException e) {
      e.printStackTrace();
    }
  }

  private FdoProfile createFdoProfile(final String id) {
    return new FdoProfile() {

      @Override
      public ProfileValidator<FDO> getValidator() {
        return new NoOpValidator();
      }

      @Override
      public String getId() {
        return id;
      }

      @Override
      public String getDescription() {
        return "Mock-Profile Description";
      }

      @Override
      public String getPid() {
        return "0.prefix/" + id;
      }
    };
  }

  @Override
  public FDO createFDO(
      FdoProfile profile, RepositoryConnection repository, Data data, Metadata metadata)
      throws ValidationException {
    FDO fdo = super.createFDO(profile, repository, data, metadata);
    synchronized (pidRegistry) {
      pidRegistry.put(fdo.getPID(), fdo);
    }
    return fdo;
  }

  @Override
  public FDO createFDO(RepositoryConnection repository, Data data, Metadata metadata)
      throws ValidationException {
    FDO fdo = super.createFDO(repository, data, metadata);
    synchronized (pidRegistry) {
      pidRegistry.put(fdo.getPID(), fdo);
    }
    return fdo;
  }

  @Override
  public DigitalObject resolvePID(String pid) {
    synchronized (pidRegistry) {
      return pidRegistry.get(pid);
    }
  }

  @Override
  public FdoProfile getDefaultProfile() {
    return getProfileRegistry().listProfiles().get(0);
  }
}
